#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#			Note:  You must reply to the verification e-mail to get through.
#
#	Copyright 2003 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Version 2.2.9
	
#Preferences

def GetPreference(pref, prefsfile):
	#Handle exceptions in parent function.
	fin = open(prefsfile, 'r')	
	text = fin.read()		
	fin.close()
	
	#Test File:
	if (text.find("drpython") < 0):
		return -1
	
	return ExtractPreferenceFromText(text, pref)

def GetPrefFromText(oldvalue, text, targetpref, integer = False):
	if (integer):
		pref = ExtractPreferenceFromText(text, targetpref)
		if (len(pref) > 0):
			return SafeInt(pref)
	else:
		pref = ExtractPreferenceFromText(text, targetpref)
		if (len(pref) > 0):
			return pref
	return oldvalue

		
def ExtractPreferenceFromText(text, pref):
	try:
		start = text.index("<" + pref + ">") + len(pref) + 2
		end = text.index("</" + pref + ">")
	
		return text[start:end]
	except:
		return ""

def ReadPreferences(prefs, prefsfile, THEME = 0):
	#Handle exceptions in parent function.
	#sets prefs for parent frame.
	#Use GetPrefFromText(oldpref, text, prefstring) for string prefs
	#Use GetPrefFromText(oldpref, text, prefstring, True) for int prefs
	#Use ExtractPreferenceFromText(text, prefstring) for prefs where and empty
	#string is a valid value.
	
	fin = open(prefsfile, 'r')	
	text = fin.read()		
	fin.close()
		
	#Test File:
	if (text.find("drpython") < 0):
		return -1
			
	#Get General Settings
	if (not THEME):	
		prefs.windowwidth = GetPrefFromText(prefs.windowwidth, text, "window.width", True)
		prefs.windowheight = GetPrefFromText(prefs.windowheight, text, "window.height", True)
		prefs.iconsize = GetPrefFromText(prefs.iconsize, text, "icon.size", True)
		prefs.mdi = GetPrefFromText(prefs.mdi, text, "mdi", True)
		prefs.enablefeedback = GetPrefFromText(prefs.enablefeedback, text, "enable.feedback", True)
		prefs.enablewarnings = GetPrefFromText(prefs.enablewarnings, text, "enable.warnings", True)
		prefs.alwayspromptonexit = GetPrefFromText(prefs.alwayspromptonexit, text, "always.prompt.on.exit", True)
		prefs.backupfileonsave = GetPrefFromText(prefs.backupfileonsave, text, "backup.file.on.save", True)
		prefs.autogotofortraceback = GetPrefFromText(prefs.autogotofortraceback, text, "auto.goto.for.traceback", True)
		
	prefs.iconfile = ExtractPreferenceFromText(text, "icon.file")	
	
	if (not THEME):	
		prefs.recentfileslimit = GetPrefFromText(prefs.recentfileslimit, text, "max.recent.files", True)
		prefs.tabwidth = GetPrefFromText(prefs.tabwidth, text, "tab.width", True)
		prefs.checkeol = GetPrefFromText(prefs.checkeol, text, "check.line.ending.type", True)
		prefs.eolmode = GetPrefFromText(prefs.eolmode, text, "line.ending.type", True)
		prefs.pythonargs = GetPrefFromText(prefs.pythonargs, text, "python.arguments")
		prefs.defaultdirectory = ExtractPreferenceFromText(text, "default.directory")
			
	#Get Document Settings
	if (not THEME):
		prefs.docmarginwidth = GetPrefFromText(prefs.docmarginwidth, text, "doc.margin.width", True)
		prefs.docwordwrap = GetPrefFromText(prefs.docwordwrap, text, "doc.wordwrap", True)
		prefs.docautoindent = GetPrefFromText(prefs.docautoindent, text, "doc.autoindent", True)
		prefs.docusetabs = GetPrefFromText(prefs.docusetabs, text, "doc.use.tabs", True)
		prefs.docwhitespaceisvisible = GetPrefFromText(prefs.docwhitespaceisvisible, text, "doc.whitespace.is.visible.on.startup", True)
		prefs.docfolding = GetPrefFromText(prefs.docfolding, text, "doc.folding", True)
		prefs.docusestyles = GetPrefFromText(prefs.docusestyles, text, "doc.use.styles", True)
		prefs.docuseindentationguides = GetPrefFromText(prefs.docuseindentationguides, text, "doc.use.indentation.guides", True)
		prefs.docscrollextrapage = GetPrefFromText(prefs.docscrollextrapage , text, "doc.scroll.extra.page", True)
	prefs.txtDocumentStyleArray[0] = GetPrefFromText(prefs.txtDocumentStyleArray[0], text, "doc.style.normal")
	prefs.txtDocumentStyleArray[1] = GetPrefFromText(prefs.txtDocumentStyleArray[1], text, "doc.style.linenumber")
	prefs.txtDocumentStyleArray[2] = GetPrefFromText(prefs.txtDocumentStyleArray[2], text, "doc.style.brace.match")
	prefs.txtDocumentStyleArray[3] = GetPrefFromText(prefs.txtDocumentStyleArray[3], text, "doc.style.brace.nomatch")
	prefs.txtDocumentStyleArray[4] = GetPrefFromText(prefs.txtDocumentStyleArray[4], text, "doc.style.character")
	prefs.txtDocumentStyleArray[5] = GetPrefFromText(prefs.txtDocumentStyleArray[5], text, "doc.style.class")
	prefs.txtDocumentStyleArray[6] = GetPrefFromText(prefs.txtDocumentStyleArray[6], text, "doc.style.comment")
	prefs.txtDocumentStyleArray[7] = GetPrefFromText(prefs.txtDocumentStyleArray[7], text, "doc.style.definition")
	prefs.txtDocumentStyleArray[8] = GetPrefFromText(prefs.txtDocumentStyleArray[8], text, "doc.style.keyword")
	prefs.txtDocumentStyleArray[9] = GetPrefFromText(prefs.txtDocumentStyleArray[9], text, "doc.style.number")
	prefs.txtDocumentStyleArray[10] = GetPrefFromText(prefs.txtDocumentStyleArray[10], text, "doc.style.operator")
	prefs.txtDocumentStyleArray[11] = GetPrefFromText(prefs.txtDocumentStyleArray[11], text, "doc.style.string")
	prefs.txtDocumentStyleArray[12] = GetPrefFromText(prefs.txtDocumentStyleArray[12], text, "doc.style.triple.string")
	prefs.txtDocumentStyleArray[13] = GetPrefFromText(prefs.txtDocumentStyleArray[13], text, "doc.style.caret")
	prefs.txtDocumentStyleArray[14] = GetPrefFromText(prefs.txtDocumentStyleArray[14], text, "doc.style.selection")
	prefs.txtDocumentStyleArray[15] = GetPrefFromText(prefs.txtDocumentStyleArray[15], text, "doc.style.folding")
	
	#Get Prompt Settings
	if (not THEME):
		prefs.promptmarginwidth = GetPrefFromText(prefs.promptmarginwidth, text, "prompt.margin.width", True)
		prefs.promptisvisible = GetPrefFromText(prefs.promptisvisible, text, "prompt.is.visible.on.startup", True)
		prefs.promptsize = GetPrefFromText(prefs.promptsize, text, "prompt.size", True)
		prefs.promptusetabs = GetPrefFromText(prefs.promptusetabs, text, "prompt.use.tabs", True)
		prefs.promptwordwrap = GetPrefFromText(prefs.promptwordwrap, text, "prompt.wordwrap", True)
		prefs.promptwhitespaceisvisible = GetPrefFromText(prefs.promptwhitespaceisvisible, text, "prompt.whitespace.is.visible.on.startup", True)
		prefs.promptusestyles = GetPrefFromText(prefs.promptusestyles, text, "prompt.use.styles", True)
		prefs.promptscrollextrapage = GetPrefFromText(prefs.promptscrollextrapage , text, "prompt.scroll.extra.page", True)
	prefs.txtPromptStyleArray[0] = GetPrefFromText(prefs.txtPromptStyleArray[0], text, "prompt.style.normal")
	prefs.txtPromptStyleArray[1] = GetPrefFromText(prefs.txtPromptStyleArray[1], text, "prompt.style.linenumber")
	prefs.txtPromptStyleArray[2] = GetPrefFromText(prefs.txtPromptStyleArray[2], text, "prompt.style.brace.match")
	prefs.txtPromptStyleArray[3] = GetPrefFromText(prefs.txtPromptStyleArray[3], text, "prompt.style.brace.nomatch")
	prefs.txtPromptStyleArray[4] = GetPrefFromText(prefs.txtPromptStyleArray[4], text, "prompt.style.character")
	prefs.txtPromptStyleArray[5] = GetPrefFromText(prefs.txtPromptStyleArray[5], text, "prompt.style.class")
	prefs.txtPromptStyleArray[6] = GetPrefFromText(prefs.txtPromptStyleArray[6], text, "prompt.style.comment")
	prefs.txtPromptStyleArray[7] = GetPrefFromText(prefs.txtPromptStyleArray[7], text, "prompt.style.definition")
	prefs.txtPromptStyleArray[8] = GetPrefFromText(prefs.txtPromptStyleArray[8], text, "prompt.style.keyword")
	prefs.txtPromptStyleArray[9] = GetPrefFromText(prefs.txtPromptStyleArray[9], text, "prompt.style.number")
	prefs.txtPromptStyleArray[10] = GetPrefFromText(prefs.txtPromptStyleArray[10], text, "prompt.style.operator")
	prefs.txtPromptStyleArray[11] = GetPrefFromText(prefs.txtPromptStyleArray[11], text, "prompt.style.string")
	prefs.txtPromptStyleArray[12] = GetPrefFromText(prefs.txtPromptStyleArray[12], text, "prompt.style.triple.string")
	prefs.txtPromptStyleArray[13] = GetPrefFromText(prefs.txtPromptStyleArray[13], text, "prompt.style.caret")
	prefs.txtPromptStyleArray[14] = GetPrefFromText(prefs.txtPromptStyleArray[14], text, "prompt.style.selection")
	
	if (not THEME):
		#Find/Replace Settings
		prefs.findreplaceregularexpression = GetPrefFromText(prefs.findreplaceregularexpression, text, "find.replace.regular.expression", True)
		prefs.findreplacematchcase = GetPrefFromText(prefs.findreplacematchcase, text, "find.replace.match.case", True)
		prefs.findreplacebackwards = GetPrefFromText(prefs.findreplacefindbackwards, text, "find.replace.find.backwards", True)
		prefs.findreplacewholeword = GetPrefFromText(prefs.findreplacewholeword, text, "find.replace.whole.word", True)
		prefs.findreplaceinselection = GetPrefFromText(prefs.findreplaceinselection, text, "find.replace.in.selection", True)
		prefs.findreplacefromcursor = GetPrefFromText(prefs.findreplacefromcursor, text, "find.replace.from.cursor", True)
		prefs.findreplacepromptonreplace = GetPrefFromText(prefs.findreplacepromptonreplace, text, "find.replace.prompt.on.replace", True)
		
		#Class Browser
		prefs.classbrowsercloseonactivate = GetPrefFromText(prefs.classbrowsercloseonactivate, text, "classbrowser.close.on.activate", True)
		prefs.classbrowserissorted = GetPrefFromText(prefs.classbrowserissorted, text, "classbrowser.is.sorted", True)
	prefs.classbrowserstyle = GetPrefFromText(prefs.classbrowserstyle, text, "classbrowser.style")
	
	#Bookmarks Settings
	prefs.bookmarksstyle = GetPrefFromText(prefs.bookmarksstyle, text, "bookmarks.style")
	
	
	#DrScript Settings
	prefs.drscriptstyle = GetPrefFromText(prefs.drscriptstyle, text, "drscript.style")
	if (not THEME):
		prefs.drscriptdefaultdirectory = ExtractPreferenceFromText(text, "drscript.default.directory")
		prefs.drscriptloadexamples = GetPrefFromText(prefs.drscriptloadexamples, text, "drscript.load.examples", True)
		
		#Theme Settings
		prefs.themesdefaultdirectory = ExtractPreferenceFromText(text, "themes.default.directory")
	
		#Printing
		prefs.printdoclinenumbers = GetPrefFromText(prefs.printdoclinenumbers, text, "print.doc.linenumbers", True)	
		prefs.printpromptlinenumbers = GetPrefFromText(prefs.printpromptlinenumbers, text, "print.prompt.linenumbers", True)
		prefs.printtabwidth = GetPrefFromText(prefs.printtabwidth, text, "print.tab.width", True)

		#Documentation
		prefs.documentationviewhelpwithbrowser = GetPrefFromText(prefs.documentationviewhelpwithbrowser, text, "documentation.view.help.with.browser", True)
		prefs.documentationbrowser = GetPrefFromText(prefs.documentationbrowser, text, "documentation.browser")
		prefs.documentationpythonlocation = GetPrefFromText(prefs.documentationpythonlocation, text, "documentation.python.location")
		prefs.documentationwxwindowslocation = GetPrefFromText(prefs.documentationwxwindowslocation, text, "documentation.wxwindows.location")
		prefs.documentationrehowtolocation = GetPrefFromText(prefs.documentationrehowtolocation, text, "documentation.rehowto.location")

def SafeInt(string):
	if (len(string) > 0):
		try:
			return int(string)
		except:
			return 0
	return 0	

def WritePreferences(prefs, prefsfile, THEME = 0):
	fin = open(prefsfile, 'w')
	
	fin.write("<drpython.version>2.2.9</drpython.version>\n")
	
	#General Settings
	if (not THEME):
		fin.write("<window.width>" + str(prefs.windowwidth) + "</window.width>\n")
		fin.write("<window.height>" + str(prefs.windowheight) + "</window.height>\n")
		fin.write("<icon.size>" + str(prefs.iconsize) + "</icon.size>\n")
		fin.write("<mdi>" + str(prefs.mdi) + "</mdi>\n")
		fin.write("<enable.feedback>" + str(prefs.enablefeedback) + "</enable.feedback>\n")
		fin.write("<enable.warnings>" + str(prefs.enablewarnings) + "</enable.warnings>\n")
		fin.write("<always.prompt.on.exit>" + str(prefs.alwayspromptonexit) + "</always.prompt.on.exit>\n")
		fin.write("<backup.file.on.save>" + str(prefs.backupfileonsave) + "</backup.file.on.save>\n")
		fin.write("<auto.goto.for.traceback>" + str(prefs.autogotofortraceback) + "</auto.goto.for.traceback>\n")
		
	fin.write("<icon.file>" + str(prefs.iconfile) + "</icon.file>\n")
	
	if (not THEME):
		fin.write("<max.recent.files>" + str(prefs.recentfileslimit) + "</max.recent.files>\n")
		fin.write("<tab.width>" + str(prefs.tabwidth) + "</tab.width>\n")
		fin.write("<check.line.ending.type>" + str(prefs.checkeol) + "</check.line.ending.type>\n")
		fin.write("<line.ending.type>" + str(prefs.eolmode) + "</line.ending.type>\n")
		fin.write("<python.arguments>" + str(prefs.pythonargs) + "</python.arguments>\n")
		fin.write("<default.directory>" + str(prefs.defaultdirectory) + "</default.directory>\n")
			
	#Document Settings
	if (not THEME):
		fin.write("<doc.margin.width>" + str(prefs.docmarginwidth) + "</doc.margin.width>\n")
		fin.write("<doc.wordwrap>" + str(prefs.docwordwrap) + "</doc.wordwrap>\n")
		fin.write("<doc.autoindent>" + str(prefs.docautoindent) + "</doc.autoindent>\n")
		fin.write("<doc.use.tabs>" + str(prefs.docusetabs) + "</doc.use.tabs>\n")
		fin.write("<doc.whitespace.is.visible.on.startup>" + str(prefs.docwhitespaceisvisible) + "</doc.whitespace.is.visible.on.startup>\n")
		fin.write("<doc.folding>" + str(prefs.docfolding) + "</doc.folding>\n")
		fin.write("<doc.use.styles>" + str(prefs.docusestyles) + "</doc.use.styles>\n")
		fin.write("<doc.use.indentation.guides>" + str(prefs.docuseindentationguides) + "</doc.use.indentation.guides>\n")
		fin.write("<doc.scroll.extra.page>" + str(prefs.docscrollextrapage) + "</doc.scroll.extra.page>\n")
	fin.write("<doc.style.normal>" + prefs.txtDocumentStyleArray[0] + "</doc.style.normal>\n")
	fin.write("<doc.style.linenumber>" + prefs.txtDocumentStyleArray[1] + "</doc.style.linenumber>\n")
	fin.write("<doc.style.brace.match>" + prefs.txtDocumentStyleArray[2] + "</doc.style.brace.match>\n")
	fin.write("<doc.style.brace.nomatch>" + prefs.txtDocumentStyleArray[3] + "</doc.style.brace.nomatch>\n")
	fin.write("<doc.style.character>" + prefs.txtDocumentStyleArray[4] + "</doc.style.character>\n")
	fin.write("<doc.style.class>" + prefs.txtDocumentStyleArray[5] + "</doc.style.class>\n")
	fin.write("<doc.style.comment>" + prefs.txtDocumentStyleArray[6] + "</doc.style.comment>\n")
	fin.write("<doc.style.definition>" + prefs.txtDocumentStyleArray[7] + "</doc.style.definition>\n")
	fin.write("<doc.style.keyword>" + prefs.txtDocumentStyleArray[8] + "</doc.style.keyword>\n")
	fin.write("<doc.style.number>" + prefs.txtDocumentStyleArray[9] + "</doc.style.number>\n")
	fin.write("<doc.style.operator>" + prefs.txtDocumentStyleArray[10] + "</doc.style.operator>\n")
	fin.write("<doc.style.string>" + prefs.txtDocumentStyleArray[11] + "</doc.style.string>\n")
	fin.write("<doc.style.triple.string>" + prefs.txtDocumentStyleArray[12] + "</doc.style.triple.string>\n")
	fin.write("<doc.style.caret>" + prefs.txtDocumentStyleArray[13] + "</doc.style.caret>\n")
	fin.write("<doc.style.selection>" + prefs.txtDocumentStyleArray[14] + "</doc.style.selection>\n")
	fin.write("<doc.style.folding>" + prefs.txtDocumentStyleArray[15] + "</doc.style.folding>\n")
	
	#Prompt Settings
	if (not THEME):
		fin.write("<prompt.margin.width>" + str(prefs.promptmarginwidth) + "</prompt.margin.width>\n")
		fin.write("<prompt.is.visible.on.startup>" + str(prefs.promptisvisible) + "</prompt.is.visible.on.startup>\n")
		fin.write("<prompt.size>" + str(prefs.promptsize) + "</prompt.size>\n")
		fin.write("<prompt.whitespace.is.visible.on.startup>" + str(prefs.promptwhitespaceisvisible) + "</prompt.whitespace.is.visible.on.startup>\n")
		fin.write("<prompt.use.tabs>" + str(prefs.promptusetabs) + "</prompt.use.tabs>\n")
		fin.write("<prompt.wordwrap>" + str(prefs.promptwordwrap) + "</prompt.wordwrap>\n")
		fin.write("<prompt.use.styles>" + str(prefs.promptusestyles) + "</prompt.use.styles>\n")
		fin.write("<prompt.scroll.extra.page>" + str(prefs.promptscrollextrapage) + "</prompt.scroll.extra.page>\n")
	fin.write("<prompt.style.normal>" + prefs.txtPromptStyleArray[0] + "</prompt.style.normal>\n")
	fin.write("<prompt.style.linenumber>" + prefs.txtPromptStyleArray[1] + "</prompt.style.linenumber>\n")
	fin.write("<prompt.style.brace.match>" + prefs.txtPromptStyleArray[2] + "</prompt.style.brace.match>\n")
	fin.write("<prompt.style.brace.nomatch>" + prefs.txtPromptStyleArray[3] + "</prompt.style.brace.nomatch>\n")
	fin.write("<prompt.style.character>" + prefs.txtPromptStyleArray[4] + "</prompt.style.character>\n")
	fin.write("<prompt.style.class>" + prefs.txtPromptStyleArray[5] + "</prompt.style.class>\n")
	fin.write("<prompt.style.comment>" + prefs.txtPromptStyleArray[6] + "</prompt.style.comment>\n")
	fin.write("<prompt.style.definition>" + prefs.txtPromptStyleArray[7] + "</prompt.style.definition>\n")
	fin.write("<prompt.style.keyword>" + prefs.txtPromptStyleArray[8] + "</prompt.style.keyword>\n")
	fin.write("<prompt.style.number>" + prefs.txtPromptStyleArray[9] + "</prompt.style.number>\n")
	fin.write("<prompt.style.operator>" + prefs.txtPromptStyleArray[10] + "</prompt.style.operator>\n")
	fin.write("<prompt.style.string>" + prefs.txtPromptStyleArray[11] + "</prompt.style.string>\n")
	fin.write("<prompt.style.triple.string>" + prefs.txtPromptStyleArray[12] + "</prompt.style.triple.string>\n")
	fin.write("<prompt.style.caret>" + prefs.txtPromptStyleArray[13] + "</prompt.style.caret>\n")
	fin.write("<prompt.style.selection>" + prefs.txtPromptStyleArray[14] + "</prompt.style.selection>\n")
	
	if (not THEME):	
		#Find/Replace Settings
		fin.write("<find.replace.regular.expression>" + str(prefs.findreplaceregularexpression) + "</find.replace.regular.expression>\n")
		fin.write("<find.replace.match.case>" + str(prefs.findreplacematchcase) + "</find.replace.match.case>\n")
		fin.write("<find.replace.find.backwards>" + str(prefs.findreplacefindbackwards) + "</find.replace.find.backwards>\n")
		fin.write("<find.replace.whole.word>" + str(prefs.findreplacewholeword ) + "</find.replace.whole.word>\n")
		fin.write("<find.replace.in.selection>" + str(prefs.findreplaceinselection) + "</find.replace.in.selection>\n")
		fin.write("<find.replace.from.cursor>" + str(prefs.findreplacefromcursor) + "</find.replace.from.cursor>\n")
		fin.write("<find.replace.prompt.on.replace>" + str(prefs.findreplacepromptonreplace) + "</find.replace.prompt.on.replace>\n")
			
		#Class Browser
		fin.write("<classbrowser.close.on.activate>" + str(prefs.classbrowsercloseonactivate) + "</classbrowser.close.on.activate>\n")
		fin.write("<classbrowser.is.sorted>" + str(prefs.classbrowserissorted) + "</classbrowser.is.sorted>\n")
	fin.write("<classbrowser.style>" + prefs.classbrowserstyle + "</classbrowser.style>\n")
	
	#Bookmarks Settings
	fin.write("<bookmarks.style>" + prefs.bookmarksstyle + "</bookmarks.style>\n")

	#DrScript Settings
	fin.write("<drscript.style>" + prefs.drscriptstyle + "</drscript.style>\n")
	if (not THEME):
		fin.write("<drscript.default.directory>" + prefs.drscriptdefaultdirectory + "</drscript.default.directory>\n")
		fin.write("<drscript.load.examples>" + str(prefs.drscriptloadexamples) + "</drscript.load.examples>\n")
		
		#Theme Settings
		fin.write("<themes.default.directory>" + prefs.themesdefaultdirectory + "</themes.default.directory>\n")
	
		#Printing
		fin.write("<print.doc.linenumbers>" + str(prefs.printdoclinenumbers) + "</print.doc.linenumbers>\n")
		fin.write("<print.prompt.linenumbers>" + str(prefs.printpromptlinenumbers) + "</print.prompt.linenumbers>\n")
		fin.write("<print.tab.width>" + str(prefs.printtabwidth) + "</print.tab.width>\n")
	
		#Documentation
		fin.write("<documentation.view.help.with.browser>" + str(prefs.documentationviewhelpwithbrowser) + "</documentation.view.help.with.browser>\n")
		fin.write("<documentation.browser>" + prefs.documentationbrowser + "</documentation.browser>\n")
		fin.write("<documentation.python.location>" + prefs.documentationpythonlocation + "</documentation.python.location>\n")
		fin.write("<documentation.wxwindows.location>" + prefs.documentationwxwindowslocation + "</documentation.wxwindows.location>\n")
		fin.write("<documentation.rehowto.location>" + prefs.documentationrehowtolocation + "</documentation.rehowto.location>\n")

	fin.close()
