#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#			Note:  You must reply to the verification e-mail to get through.
#
#	Copyright 2003 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Version 2.2.9
	
#Shortcuts File

import drPrefsFile
from drShortcuts import drShortcut

def GetShortcutList():
	#Shortcut List
	return ["New", "Open", "Save", "Save As", "Save Prompt Output To File", "Reload File", \
		"Restore From Backup", "Close", \
		"Print Setup", "Print File", "Print Prompt", "Exit", \
		"Next Tab", "Previous Tab", "First Tab", "Last Tab", \
		"Find", "Find Next", "Find Previous", \
		"Replace", "Switcheroo", "Insert Regular Expression", "Go To", \
		"Comment", "UnComment", "Indent", "Dedent", \
		"Uppercase", "Lowercase", "Undo", "Redo", "Copy", "Cut", "Paste", \
		"Select All", "Select None", "Zoom In", "Zoom Out", "Fold All", "Expand All", \
		"Show Class Browser", "Toggle View Whitespace", "Toggle Prompt", "Clear Prompt", \
		"Run", "Set Arguments", "Python", "Run With Debugger", "End", \
		"Set Log File", "Insert Logger", "Insert String Logger", "Insert Arbitrary Arguments Logger", \
		"Remove All Loggers", \
		"Add Breakpoint", "Remove Breakpoint", "Remove All Breakpoints", "Preferences", \
		"Help", "View Python Docs", "View WxWindows Docs", "View Regular Expression Howto", \
		"Toggle Maximize"]

def IsBuiltIn(name):
	return 1

def IsNotAtLeastVersion(numberString, targetString):
	if len(numberString) < 1:
		return 1
	na = numberString.find('.')
	nb = na + 1 + numberString[na+1:].find('.')
	nc = len(numberString)
	
	ta = targetString.find('.')
	tb = ta + 1 + targetString[ta+1:].find('.')
	tc = len(targetString)
	
	try:
		nvA = int(numberString[:na])
		nvB = int(numberString[na+1:nb])
		nvC = int(numberString[nb+1:nc])
	
		tvA = int(targetString[:ta])
		tvB = int(targetString[ta+1:tb])
		tvC = int(targetString[tb+1:tc])
		
		if nvA < tvA:
			return 1
		elif nvB < tvB:
			return 1
		elif nvC < tvC:
			return 1
	except:
		return 1
		
	return 0

def GetDefaultShortcuts():
	shortcutArray = []
	keycodeArray = []	
	
	import sys
	if (sys.platform.find("linux") > -1):
		ignorestring = "Meta,"
	else:
		ignorestring = ""
	
	ShortcutList = GetShortcutList()
	l = len(ShortcutList)
	x = 0
	while (x < l):
		shortcutArray.append(drShortcut())
		shortcutArray[x].Name = ShortcutList[x]
		keycodeArray.append("")		
		x = x + 1
	
	return shortcutArray, keycodeArray, ignorestring

def ReadShortcuts(filename, getIgnoreString = 1):
	f = open(filename, 'r')
	
	shortcutArray = []
	keycodeArray = []
	ignorestring = ""
	
	if (getIgnoreString):
		#Get Version number first
		line = f.readline()
		versionstring = drPrefsFile.ExtractPreferenceFromText(line, "version.number")
		if IsNotAtLeastVersion(versionstring, "2.2.0"):
			#Automatically Update Shortcuts File.
			f.close()
			UpdateShortcutsFile(filename, versionstring)			
			return ReadShortcuts(filename)			
		#Get ignorestring second
		line = f.readline()
		ignorestring = drPrefsFile.ExtractPreferenceFromText(line, "ignore")
	
	line = f.readline()
	x = 0
	while (len(line) > 0):
		shortcutArray.append(drShortcut())
		shortcutArray[x].Name = drPrefsFile.ExtractPreferenceFromText(line, "shortcut")
		shortcutArray[x].Keycode = drPrefsFile.ExtractPreferenceFromText(line, "keycode")
		
		keycodeArray.append(shortcutArray[x].Keycode)
		
		modifiers = drPrefsFile.ExtractPreferenceFromText(line, "mod")
		shortcutArray[x].Control = (modifiers.find("Control") > -1)
		shortcutArray[x].Shift = (modifiers.find("Shift") > -1)
		shortcutArray[x].Meta = (modifiers.find("Meta") > -1)
		shortcutArray[x].Alt = (modifiers.find("Alt") > -1)		
		x = x + 1
		line = f.readline()
	f.close()
	
	return shortcutArray, keycodeArray, ignorestring

def UpdateShortcutsFile(filename, oldversion):
	f = open(filename, 'r')
	text = f.read()
	f.close()
	
	if (len(text) < 1):
		#What a corrupt file!
		import os
		os.remove(filename)
		return
	
	if len(oldversion) < 1:
		#This file is pre 2.2.0
		text = "<version.number>2.2.9</version.number>\n" + text 
		i = text.find("Save As")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Save Prompt Output To File</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		i = text.find("Reload File")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Restore From Backup</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		i = text.find("Exit")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Next Tab</shortcut><mod></mod><keycode>351</keycode>\n"\
		 + "<shortcut>Previous Tab</shortcut><mod></mod><keycode>350</keycode>\n"\
		 + "<shortcut>First Tab</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Last Tab</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		i = text.find("Find Next")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Find Previous</shortcut><mod>Shift,</mod><keycode>344</keycode>\n"\
		+ text[i:]
		
		i = text.find("Switcheroo")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Insert Regular Expression</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		i = text.find("End")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Set Log File</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Insert Logger</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Insert String Logger</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Insert Arbitrary Arguments Logger</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Remove All Loggers</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		i = text.find("Remove All Breakpoints")
		i = i + text[i:].find('\n') + 1
		text = text[:i] + "<shortcut>Preferences</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>Help</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>View Python Docs</shortcut><mod></mod><keycode>349</keycode>\n"\
		 + "<shortcut>View WxWindows Docs</shortcut><mod></mod><keycode></keycode>\n"\
		 + "<shortcut>View Regular Expression Howto</shortcut><mod></mod><keycode></keycode>\n"\
		+ text[i:]
		
		f = open(filename, 'w')
		f.write(text)
		f.close()
	

def WriteShortcuts(filename, shortcutlist, txtValueArray, ignorestring = "", startAt = 0):

	f = open(filename, 'w')
	
	if (not startAt):
		#Ignore String
		f.write("<version.number>2.2.9</version.number>\n")
		f.write("<ignore>" + ignorestring + "</ignore>\n")
	allowControl = not (ignorestring.find("Control") > -1)
	allowShift = not (ignorestring.find("Shift") > -1)
	allowMeta = not (ignorestring.find("Meta") > -1)
	allowAlt = not (ignorestring.find("Alt") > -1)	
	
	l = len(shortcutlist)
	x = startAt
	y = 0
	while (y < l):
		value = txtValueArray[x].GetValue()
		writestring = "<shortcut>" + shortcutlist[y] + "</shortcut><mod>"
		target = value.find("#")
		if (target > -1):
			if (value.find("Control") > -1) and allowControl:
				writestring = writestring + "Control,"
			if (value.find("Shift") > -1) and allowShift:
				writestring = writestring + "Shift,"
			if (value.find("Meta") > -1) and allowMeta:
				writestring = writestring + "Meta,"
			if (value.find("Alt") > -1) and allowAlt:
				writestring = writestring + "Alt,"
		writestring = writestring + "</mod><keycode>" + value[(target + 1):] + "</keycode>\n"
		f.write(writestring)
		x = x + 1
		y = y + 1
	f.close()
	
def WriteShortcutsFromText(filename, shortcutarray, ignorestring = ""):

	f = open(filename, 'w')
	
	allowControl = not (ignorestring.find("Control") > -1)
	allowShift = not (ignorestring.find("Shift") > -1)
	allowMeta = not (ignorestring.find("Meta") > -1)
	allowAlt = not (ignorestring.find("Alt") > -1)	
	
	l = len(shortcutarray)
	x = 0
	while (x < l):
		writestring = "<shortcut>" + shortcutarray[x].Name + "</shortcut><mod>"
		if (shortcutarray[x].Control) and allowControl:
			writestring = writestring + "Control,"
		if (shortcutarray[x].Shift) and allowShift:
			writestring = writestring + "Shift,"
		if (shortcutarray[x].Meta) and allowMeta:
			writestring = writestring + "Meta,"
		if (shortcutarray[x].Alt) and allowAlt:
			writestring = writestring + "Alt,"
		writestring = writestring + "</mod><keycode>" + str(shortcutarray[x].Keycode) + "</keycode>\n"
		f.write(writestring)
		x = x + 1
		
	f.close()
	
